!
!
!                             AMG4PSBLAS version 1.0
!    Algebraic Multigrid Package
!               based on PSBLAS (Parallel Sparse BLAS version 3.7)
!
!    (C) Copyright 2021
!
!        Salvatore Filippone
!        Pasqua D'Ambra
!        Daniela di Serafino
!
!    Redistribution and use in source and binary forms, with or without
!    modification, are permitted provided that the following conditions
!    are met:
!      1. Redistributions of source code must retain the above copyright
!         notice, this list of conditions and the following disclaimer.
!      2. Redistributions in binary form must reproduce the above copyright
!         notice, this list of conditions, and the following disclaimer in the
!         documentation and/or other materials provided with the distribution.
!      3. The name of the AMG4PSBLAS group or the names of its contributors may
!         not be used to endorse or promote products derived from this
!         software without specific written permission.
!
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
!    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
!    TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
!    PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AMG4PSBLAS GROUP OR ITS CONTRIBUTORS
!    BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
!    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
!    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
!    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
!    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
!    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
!    POSSIBILITY OF SUCH DAMAGE.
!
!
!
! File: amg_d_poly_smoother_mod.f90
!
! Module: amg_d_poly_smoother_mod
!
!  This module defines:
!    the amg_d_poly_smoother_type data structure containing the
!    smoother for a Jacobi/block Jacobi smoother.
!  The smoother stores in ND the block off-diagonal matrix.
!  One special case is treated separately, when the solver is DIAG or L1-DIAG
!  then the ND is the entire off-diagonal part of the matrix (including the
!  main diagonal block), so that it becomes possible to implement
!  a pure Jacobi or L1-Jacobi global solver.
!
module amg_d_poly_coeff_mod
  use psb_base_mod

  real(psb_dpk_), parameter :: amg_d_poly_a_vect(30) = [ &
       & 0.3333333333333333_psb_dpk_, &
       & 0.1805359927403007_psb_dpk_, &
       & 0.1159278464862213_psb_dpk_, &
       & 0.0820780659590383_psb_dpk_, &
       & 0.0618496002413377_psb_dpk_, &
       & 0.0486605823426062_psb_dpk_, &
       & 0.0395132986024057_psb_dpk_, &
       & 0.0328701017544880_psb_dpk_, &
       & 0.0278702862721800_psb_dpk_, &
       & 0.0239987409600620_psb_dpk_, &
       & 0.0209304400432259_psb_dpk_, &
       & 0.0184513099045066_psb_dpk_, &
       & 0.0164152586042591_psb_dpk_, &
       & 0.0147195638076874_psb_dpk_, &
       & 0.0132901324757843_psb_dpk_, &
       & 0.0120723317737698_psb_dpk_, &
       & 0.0110250964606384_psb_dpk_, &
       & 0.0101170330064859_psb_dpk_, &
       & 0.0093237789039835_psb_dpk_, &
       & 0.0086261728849515_psb_dpk_, &
       & 0.0080089618703679_psb_dpk_, &
       & 0.0074598709610601_psb_dpk_, &
       & 0.0069689238144320_psb_dpk_, &
       & 0.0065279387776372_psb_dpk_, &
       & 0.0061301503808627_psb_dpk_, &
       & 0.0057699215598864_psb_dpk_, &
       & 0.0054425224281914_psb_dpk_, &
       & 0.0051439584672521_psb_dpk_, &
       & 0.0048708358327268_psb_dpk_, &
       & 0.0046202548314912_psb_dpk_ ];
  
  real(psb_dpk_), parameter :: amg_d_poly_beta_vect(900) = [ &
       & 1.1250000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0238728757031315_psb_dpk_, 1.2640890537108553_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0084254478202830_psb_dpk_, 1.0886783920873087_psb_dpk_, &
       & 1.3375312590961856_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0039131042728535_psb_dpk_, 1.0403581118859304_psb_dpk_, &
       & 1.1486349854625493_psb_dpk_, 1.3826886924100055_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0021293014616472_psb_dpk_, 1.0217371154926094_psb_dpk_, &
       & 1.0787243319260302_psb_dpk_, 1.1981006529266300_psb_dpk_, &
       & 1.4132254279168215_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0012851725594023_psb_dpk_, 1.0130429303523338_psb_dpk_, &
       & 1.0467821512411335_psb_dpk_, 1.1161648941967548_psb_dpk_, &
       & 1.2382902021844453_psb_dpk_, 1.4352429710674484_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0008346439791242_psb_dpk_, 1.0084394943012289_psb_dpk_, &
       & 1.0300870776871385_psb_dpk_, 1.0740838409200377_psb_dpk_, &
       & 1.1503618670736642_psb_dpk_, 1.2711647404613990_psb_dpk_, &
       & 1.4518665864936395_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0005724663119766_psb_dpk_, 1.0057742766241562_psb_dpk_, &
       & 1.0205018792294143_psb_dpk_, 1.0501980344456543_psb_dpk_, &
       & 1.1011557298494106_psb_dpk_, 1.1808604280685657_psb_dpk_, &
       & 1.2983858538257604_psb_dpk_, 1.4648607315109978_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0004096007283281_psb_dpk_, 1.0041243950610661_psb_dpk_, &
       & 1.0146021214826659_psb_dpk_, 1.0356111362667175_psb_dpk_, &
       & 1.0713997252919425_psb_dpk_, 1.1268827371096291_psb_dpk_, &
       & 1.2078521914072933_psb_dpk_, 1.3212193071674674_psb_dpk_, &
       & 1.4752964282069962_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0003031222965291_psb_dpk_, 1.0030484066079688_psb_dpk_, &
       & 1.0107702271538761_psb_dpk_, 1.0261901159764004_psb_dpk_, &
       & 1.0523172493375519_psb_dpk_, 1.0925574320754976_psb_dpk_, &
       & 1.1508337666397197_psb_dpk_, 1.2317225087089441_psb_dpk_, &
       & 1.3406080202445980_psb_dpk_, 1.4838612440701109_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0002305859520939_psb_dpk_, 1.0023167502402850_psb_dpk_, &
       & 1.0081724539630488_psb_dpk_, 1.0198298656634219_psb_dpk_, &
       & 1.0395021023532465_psb_dpk_, 1.0696504270054137_psb_dpk_, &
       & 1.1130575429574259_psb_dpk_, 1.1729087627556418_psb_dpk_, &
       & 1.2528830057679230_psb_dpk_, 1.3572557991951903_psb_dpk_, &
       & 1.4910167256413891_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0001794720082837_psb_dpk_, 1.0018018913961957_psb_dpk_, &
       & 1.0063486190730762_psb_dpk_, 1.0153786456630600_psb_dpk_, &
       & 1.0305694283076039_psb_dpk_, 1.0537601969394355_psb_dpk_, &
       & 1.0869986259207296_psb_dpk_, 1.1325918309791341_psb_dpk_, &
       & 1.1931627335817252_psb_dpk_, 1.2717129367511055_psb_dpk_, &
       & 1.3716933796979953_psb_dpk_, 1.4970841857556243_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0001424192155957_psb_dpk_, 1.0014290693262966_psb_dpk_, &
       & 1.0050302898629815_psb_dpk_, 1.0121691051849540_psb_dpk_, &
       & 1.0241487434279255_psb_dpk_, 1.0423815888082042_psb_dpk_, &
       & 1.0684200812870084_psb_dpk_, 1.1039901093675994_psb_dpk_, &
       & 1.1510274824264566_psb_dpk_, 1.2117181191012512_psb_dpk_, &
       & 1.2885426486512805_psb_dpk_, 1.3843261938099158_psb_dpk_, &
       & 1.5022941875736890_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0001149053826193_psb_dpk_, 1.0011524637691460_psb_dpk_, &
       & 1.0040535733326481_psb_dpk_, 1.0097959057315313_psb_dpk_, &
       & 1.0194130047299461_psb_dpk_, 1.0340142503543679_psb_dpk_, &
       & 1.0548059960662932_psb_dpk_, 1.0831142030181304_psb_dpk_, &
       & 1.1204089166089239_psb_dpk_, 1.1683309565544606_psb_dpk_, &
       & 1.2287212228823874_psb_dpk_, 1.3036530570781755_psb_dpk_, &
       & 1.3954681405367855_psb_dpk_, 1.5068164620958386_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000940475075257_psb_dpk_, 1.0009429169634352_psb_dpk_, &
       & 1.0033144905644482_psb_dpk_, 1.0080029483381612_psb_dpk_, &
       & 1.0158423625914039_psb_dpk_, 1.0277208331770495_psb_dpk_, &
       & 1.0445953542283146_psb_dpk_, 1.0675076120612534_psb_dpk_, &
       & 1.0976009254588965_psb_dpk_, 1.1361385536615733_psb_dpk_, &
       & 1.1845236142623621_psb_dpk_, 1.2443208730447588_psb_dpk_, &
       & 1.3172806908339272_psb_dpk_, 1.4053654389356023_psb_dpk_, &
       & 1.5107787250184523_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000779482817921_psb_dpk_, 1.0007812684725339_psb_dpk_, &
       & 1.0027448797440124_psb_dpk_, 1.0066229101701514_psb_dpk_, &
       & 1.0130985883697137_psb_dpk_, 1.0228944832933697_psb_dpk_, &
       & 1.0367832140998394_psb_dpk_, 1.0555987571989653_psb_dpk_, &
       & 1.0802484840556024_psb_dpk_, 1.1117260713149764_psb_dpk_, &
       & 1.1511254343107276_psb_dpk_, 1.1996558461497355_psb_dpk_, &
       & 1.2586584174494597_psb_dpk_, 1.3296241265666493_psb_dpk_, &
       & 1.4142136069557629_psb_dpk_, 1.5142789173034623_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000653242183546_psb_dpk_, 1.0006545722939437_psb_dpk_, &
       & 1.0022987777448662_psb_dpk_, 1.0055432691173583_psb_dpk_, &
       & 1.0109550075016893_psb_dpk_, 1.0191301541168694_psb_dpk_, &
       & 1.0307019481191382_psb_dpk_, 1.0463489778000818_psb_dpk_, &
       & 1.0668039321569163_psb_dpk_, 1.0928629244731740_psb_dpk_, &
       & 1.1253954850882542_psb_dpk_, 1.1653553270075827_psb_dpk_, &
       & 1.2137919954743157_psb_dpk_, 1.2718635211544003_psb_dpk_, &
       & 1.3408502062615073_psb_dpk_, 1.4221696838526183_psb_dpk_, &
       & 1.5173934027630227_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000552858792859_psb_dpk_, 1.0005538659610900_psb_dpk_, &
       & 1.0019444166743086_psb_dpk_, 1.0046864301776393_psb_dpk_, &
       & 1.0092557508630260_psb_dpk_, 1.0161502674772371_psb_dpk_, &
       & 1.0258958148322650_psb_dpk_, 1.0390523408953256_psb_dpk_, &
       & 1.0562203973533295_psb_dpk_, 1.0780480145522537_psb_dpk_, &
       & 1.1052380250439366_psb_dpk_, 1.1385559038570177_psb_dpk_, &
       & 1.1788381980793483_psb_dpk_, 1.2270016234308427_psb_dpk_, &
       & 1.2840529112630572_psb_dpk_, 1.3510994958895055_psb_dpk_, &
       & 1.4293611393851839_psb_dpk_, 1.5201825990516680_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000472036358790_psb_dpk_, 1.0004728102642675_psb_dpk_, &
       & 1.0016593577469159_psb_dpk_, 1.0039976891368516_psb_dpk_, &
       & 1.0078911941833455_psb_dpk_, 1.0137601583069535_psb_dpk_, &
       & 1.0220462561721002_psb_dpk_, 1.0332172281153209_psb_dpk_, &
       & 1.0477717791157513_psb_dpk_, 1.0662447417325256_psb_dpk_, &
       & 1.0892125464929936_psb_dpk_, 1.1172990456131733_psb_dpk_, &
       & 1.1511817386833911_psb_dpk_, 1.1915984520803475_psb_dpk_, &
       & 1.2393545273929878_psb_dpk_, 1.2953305781018039_psb_dpk_, &
       & 1.3604908781568688_psb_dpk_, 1.4358924509939206_psb_dpk_, &
       & 1.5226949329440265_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000406232569254_psb_dpk_, 1.0004068351374691_psb_dpk_, &
       & 1.0014274431564170_psb_dpk_, 1.0034377175807407_psb_dpk_, &
       & 1.0067826854070978_psb_dpk_, 1.0118204999571436_psb_dpk_, &
       & 1.0189259121271075_psb_dpk_, 1.0284938700470616_psb_dpk_, &
       & 1.0409432748132981_psb_dpk_, 1.0567209210598594_psb_dpk_, &
       & 1.0763056524407055_psb_dpk_, 1.1002127636100871_psb_dpk_, &
       & 1.1289986820268283_psb_dpk_, 1.1632659648787138_psb_dpk_, &
       & 1.2036686486408621_psb_dpk_, 1.2509179912601627_psb_dpk_, &
       & 1.3057886497146727_psb_dpk_, 1.3691253387497200_psb_dpk_, &
       & 1.4418500199624611_psb_dpk_, 1.5249696741164267_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000352114440929_psb_dpk_, 1.0003525892395289_psb_dpk_, &
       & 1.0012368357172980_psb_dpk_, 1.0029777430511673_psb_dpk_, &
       & 1.0058727830027672_psb_dpk_, 1.0102297507781717_psb_dpk_, &
       & 1.0163694815733537_psb_dpk_, 1.0246286588536329_psb_dpk_, &
       & 1.0353627340015590_psb_dpk_, 1.0489489776835172_psb_dpk_, &
       & 1.0657896841306789_psb_dpk_, 1.0863155505114006_psb_dpk_, &
       & 1.1109892546943501_psb_dpk_, 1.1403092559728156_psb_dpk_, &
       & 1.1748138447471401_psb_dpk_, 1.2150854687543668_psb_dpk_, &
       & 1.2617553651999671_psb_dpk_, 1.3155085300984379_psb_dpk_, &
       & 1.3770890582780710_psb_dpk_, 1.4473058898645985_psb_dpk_, &
       & 1.5270390016420912_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000307198714835_psb_dpk_, 1.0003075769178242_psb_dpk_, &
       & 1.0010787281022711_psb_dpk_, 1.0025963829693492_psb_dpk_, &
       & 1.0051188625231162_psb_dpk_, 1.0089126974249720_psb_dpk_, &
       & 1.0142547789760521_psb_dpk_, 1.0214345766593154_psb_dpk_, &
       & 1.0307564364069204_psb_dpk_, 1.0425419742322541_psb_dpk_, &
       & 1.0571325804249445_psb_dpk_, 1.0748920501551993_psb_dpk_, &
       & 1.0962093570737961_psb_dpk_, 1.1215015873309027_psb_dpk_, &
       & 1.1512170523743910_psb_dpk_, 1.1858385999327761_psb_dpk_, &
       & 1.2258871437439198_psb_dpk_, 1.2719254338660289_psb_dpk_, &
       & 1.3245620908078453_psb_dpk_, 1.3844559282498121_psb_dpk_, &
       & 1.4523205908039656_psb_dpk_, 1.5289295350887884_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000269609460124_psb_dpk_, 1.0002699137181752_psb_dpk_, &
       & 1.0009464748475532_psb_dpk_, 1.0022775198638552_psb_dpk_, &
       & 1.0044888368184179_psb_dpk_, 1.0078128087804721_psb_dpk_, &
       & 1.0124901352066715_psb_dpk_, 1.0187716022931539_psb_dpk_, &
       & 1.0269199126829005_psb_dpk_, 1.0372115852204526_psb_dpk_, &
       & 1.0499389358225151_psb_dpk_, 1.0654121509688057_psb_dpk_, &
       & 1.0839614658147161_psb_dpk_, 1.1059394594887115_psb_dpk_, &
       & 1.1317234807654135_psb_dpk_, 1.1617182180038959_psb_dpk_, &
       & 1.1963584280123116_psb_dpk_, 1.2361118393501820_psb_dpk_, &
       & 1.2814822465106404_psb_dpk_, 1.3330128124440397_psb_dpk_, &
       & 1.3912895979940381_psb_dpk_, 1.4569453380258381_psb_dpk_, &
       & 1.5306634853375161_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000237911597230_psb_dpk_, 1.0002381585998457_psb_dpk_, &
       & 1.0008349974382460_psb_dpk_, 1.0020088476285827_psb_dpk_, &
       & 1.0039582343156432_psb_dpk_, 1.0068870298152559_psb_dpk_, &
       & 1.0110058445931565_psb_dpk_, 1.0165334547611182_psb_dpk_, &
       & 1.0236982737890488_psb_dpk_, 1.0327398763510158_psb_dpk_, &
       & 1.0439105824804926_psb_dpk_, 1.0574771105088172_psb_dpk_, &
       & 1.0737223076000839_psb_dpk_, 1.0929469670793606_psb_dpk_, &
       & 1.1154717421787756_psb_dpk_, 1.1416391663018148_psb_dpk_, &
       & 1.1718157904303341_psb_dpk_, 1.2063944488757254_psb_dpk_, &
       & 1.2457966652063013_psb_dpk_, 1.2904752108716941_psb_dpk_, &
       & 1.3409168297942540_psb_dpk_, 1.3976451430108305_psb_dpk_, &
       & 1.4612237483301715_psb_dpk_, 1.5322595309246121_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000210994601235_psb_dpk_, 1.0002111968041199_psb_dpk_, &
       & 1.0007403694573151_psb_dpk_, 1.0017808593384865_psb_dpk_, &
       & 1.0035081686576977_psb_dpk_, 1.0061021720448531_psb_dpk_, &
       & 1.0097482505685551_psb_dpk_, 1.0146384533048582_psb_dpk_, &
       & 1.0209726922414943_psb_dpk_, 1.0289599764553270_psb_dpk_, &
       & 1.0388196916802268_psb_dpk_, 1.0507829315895938_psb_dpk_, &
       & 1.0650938873538003_psb_dpk_, 1.0820113022982043_psb_dpk_, &
       & 1.1018099987843295_psb_dpk_, 1.1247824847650900_psb_dpk_, &
       & 1.1512406478277994_psb_dpk_, 1.1815175449359154_psb_dpk_, &
       & 1.2159692965153148_psb_dpk_, 1.2549770940040335_psb_dpk_, &
       & 1.2989493304988182_psb_dpk_, 1.3483238646890843_psb_dpk_, &
       & 1.4035704288718982_psb_dpk_, 1.4651931924923849_psb_dpk_, &
       & 1.5337334933563860_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000187989989242_psb_dpk_, 1.0001881567984481_psb_dpk_, &
       & 1.0006595227084085_psb_dpk_, 1.0015861311895899_psb_dpk_, &
       & 1.0031239047778964_psb_dpk_, 1.0054323694760092_psb_dpk_, &
       & 1.0086755868504005_psb_dpk_, 1.0130231071421940_psb_dpk_, &
       & 1.0186509477893992_psb_dpk_, 1.0257426018654052_psb_dpk_, &
       & 1.0344900810652515_psb_dpk_, 1.0450949980170887_psb_dpk_, &
       & 1.0577696928624343_psb_dpk_, 1.0727384092356933_psb_dpk_, &
       & 1.0902385249817814_psb_dpk_, 1.1105218431816117_psb_dpk_, &
       & 1.1338559493090710_psb_dpk_, 1.1605256406217599_psb_dpk_, &
       & 1.1908344341913664_psb_dpk_, 1.2251061603103259_psb_dpk_, &
       & 1.2636866483695495_psb_dpk_, 1.3069455126904677_psb_dpk_, &
       & 1.3552780462128098_psb_dpk_, 1.4091072303921326_psb_dpk_, &
       & 1.4688858701459975_psb_dpk_, 1.5350988632115488_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000168211938973_psb_dpk_, 1.0001683505351420_psb_dpk_, &
       & 1.0005900360142315_psb_dpk_, 1.0014188084960041_psb_dpk_, &
       & 1.0027938311393803_psb_dpk_, 1.0048572584314193_psb_dpk_, &
       & 1.0077550080990554_psb_dpk_, 1.0116375492127350_psb_dpk_, &
       & 1.0166607098595459_psb_dpk_, 1.0229865078405374_psb_dpk_, &
       & 1.0307840079371537_psb_dpk_, 1.0402302093961155_psb_dpk_, &
       & 1.0515109674005423_psb_dpk_, 1.0648219524284319_psb_dpk_, &
       & 1.0803696515480321_psb_dpk_, 1.0983724158638981_psb_dpk_, &
       & 1.1190615585080472_psb_dpk_, 1.1426825077681895_psb_dpk_, &
       & 1.1694960201606786_psb_dpk_, 1.1997794584895700_psb_dpk_, &
       & 1.2338281401870808_psb_dpk_, 1.2719567615042522_psb_dpk_, &
       & 1.3145009034164739_psb_dpk_, 1.3618186254259919_psb_dpk_, &
       & 1.4142921537855777_psb_dpk_, 1.4723296710339275_psb_dpk_, &
       & 1.5363672141264497_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000151113991291_psb_dpk_, 1.0001512299115287_psb_dpk_, &
       & 1.0005299814085029_psb_dpk_, 1.0012742317597600_psb_dpk_, &
       & 1.0025087130476142_psb_dpk_, 1.0043606572645858_psb_dpk_, &
       & 1.0069604400315522_psb_dpk_, 1.0104422369100252_psb_dpk_, &
       & 1.0149446949285030_psb_dpk_, 1.0206116219981500_psb_dpk_, &
       & 1.0275926969588451_psb_dpk_, 1.0360442030716124_psb_dpk_, &
       & 1.0461297878595799_psb_dpk_, 1.0580212522952626_psb_dpk_, &
       & 1.0718993724396861_psb_dpk_, 1.0879547567564958_psb_dpk_, &
       & 1.1063887424550545_psb_dpk_, 1.1274143343577541_psb_dpk_, &
       & 1.1512571899424711_psb_dpk_, 1.1781566543781672_psb_dpk_, &
       & 1.2083668495540898_psb_dpk_, 1.2421578212983135_psb_dpk_, &
       & 1.2798167491932815_psb_dpk_, 1.3216492236219661_psb_dpk_, &
       & 1.3679805949228399_psb_dpk_, 1.4191573997915068_psb_dpk_, &
       & 1.4755488703473389_psb_dpk_, 1.5375485315807513_psb_dpk_, &
       & 0.0000000000000000_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000136257096588_psb_dpk_, 1.0001363546506836_psb_dpk_, &
       & 1.0004778107488095_psb_dpk_, 1.0011486612681773_psb_dpk_, &
       & 1.0022611433613271_psb_dpk_, 1.0039295964948667_psb_dpk_, &
       & 1.0062710027404669_psb_dpk_, 1.0094055369479136_psb_dpk_, &
       & 1.0134571288503909_psb_dpk_, 1.0185540391932908_psb_dpk_, &
       & 1.0248294520252528_psb_dpk_, 1.0324220853457433_psb_dpk_, &
       & 1.0414768223656390_psb_dpk_, 1.0521453657079123_psb_dpk_, &
       & 1.0645869169533493_psb_dpk_, 1.0789688840227822_psb_dpk_, &
       & 1.0954676189818162_psb_dpk_, 1.1142691889576817_psb_dpk_, &
       & 1.1355701829701565_psb_dpk_, 1.1595785576006521_psb_dpk_, &
       & 1.1865145245551894_psb_dpk_, 1.2166114833191515_psb_dpk_, &
       & 1.2501170022543431_psb_dpk_, 1.2872938516530203_psb_dpk_, &
       & 1.3284210924391027_psb_dpk_, 1.3737952243949607_psb_dpk_, &
       & 1.4237313979931023_psb_dpk_, 1.4785646941265451_psb_dpk_, &
       & 1.5386514762605854_psb_dpk_, 0.0000000000000000_psb_dpk_, &
       & 1.0000123285767939_psb_dpk_, 1.0001233683396147_psb_dpk_, &
       & 1.0004322711781202_psb_dpk_, 1.0010390719329101_psb_dpk_, &
       & 1.0020451337350940_psb_dpk_, 1.0035535979966428_psb_dpk_, &
       & 1.0056698406248343_psb_dpk_, 1.0085019360540697_psb_dpk_, &
       & 1.0121611307132341_psb_dpk_, 1.0167623275769953_psb_dpk_, &
       & 1.0224245834847208_psb_dpk_, 1.0292716209515502_psb_dpk_, &
       & 1.0374323562422998_psb_dpk_, 1.0470414455308106_psb_dpk_, &
       & 1.0582398510249318_psb_dpk_, 1.0711754290010183_psb_dpk_, &
       & 1.0860035417614331_psb_dpk_, 1.1028876956049132_psb_dpk_, &
       & 1.1220002069820316_psb_dpk_, 1.1435228990979547_psb_dpk_, &
       & 1.1676478313209715_psb_dpk_, 1.1945780638597872_psb_dpk_, &
       & 1.2245284602839432_psb_dpk_, 1.2577265305821996_psb_dpk_, &
       & 1.2944133175813315_psb_dpk_, 1.3348443296857557_psb_dpk_, &
       & 1.3792905230439911_psb_dpk_, 1.4280393364047606_psb_dpk_, &
       & 1.4813957820911738_psb_dpk_, 1.5396835966986973_psb_dpk_ ]




!!$  [1.1250000000000000_psb_dpk_, 0.0_psb_dpk_, 0.0_psb_dpk__psb_dpk_,,&
!!$       & 1.0238728757031315_psb_dpk_, 1.2640890537108553_psb_dpk_,  0.0_psb_dpk_,&
!!$       & 1.0084254478202830_psb_dpk_, 1.0886783920873087_psb_dpk_, 1.3375312590961856_psb_dpk_]

  real(psb_dpk_), parameter  :: amg_d_poly_beta_mat(30,30)=reshape(amg_d_poly_beta_vect,[30,30])

end module amg_d_poly_coeff_mod
