!
!
!                             AMG4PSBLAS version 1.0
!    Algebraic Multigrid Package
!               based on PSBLAS (Parallel Sparse BLAS version 3.7)
!
!    (C) Copyright 2021
!
!        Salvatore Filippone
!        Pasqua D'Ambra
!        Daniela di Serafino
!
!    Redistribution and use in source and binary forms, with or without
!    modification, are permitted provided that the following conditions
!    are met:
!      1. Redistributions of source code must retain the above copyright
!         notice, this list of conditions and the following disclaimer.
!      2. Redistributions in binary form must reproduce the above copyright
!         notice, this list of conditions, and the following disclaimer in the
!         documentation and/or other materials provided with the distribution.
!      3. The name of the AMG4PSBLAS group or the names of its contributors may
!         not be used to endorse or promote products derived from this
!         software without specific written permission.
!
!    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
!    ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
!    TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
!    PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AMG4PSBLAS GROUP OR ITS CONTRIBUTORS
!    BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
!    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
!    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
!    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
!    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
!    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
!    POSSIBILITY OF SUCH DAMAGE.
!
!
subroutine amg_s_poly_smoother_cseti(sm,what,val,info,idx)

  use psb_base_mod
  use amg_s_poly_smoother, amg_protect_nam => amg_s_poly_smoother_cseti
  Implicit None

  ! Arguments
  class(amg_s_poly_smoother_type), intent(inout) :: sm
  character(len=*), intent(in)                 :: what
  integer(psb_ipk_), intent(in)                :: val
  integer(psb_ipk_), intent(out)               :: info
  integer(psb_ipk_), intent(in), optional      :: idx
  integer(psb_ipk_) :: err_act
  character(len=20)  :: name='d_poly_smoother_cseti'

  info = psb_success_
  call psb_erractionsave(err_act)

  select case(psb_toupper(what))
  case('POLY_DEGREE')
    sm%pdegree = val
  case('POLY_VARIANT')
    select case(val)
    case(amg_cheb_4_,amg_cheb_4_opt_,amg_cheb_1_opt_)
      sm%variant = val
    case default
      write(0,*) 'Invalid choice for POLY_VARIANT, defaulting to amg_cheb_4_',val
      sm%variant = amg_cheb_4_
    end select
  case('POLY_RHO_ESTIMATE')
    select case(val)
    case (amg_poly_rho_est_power_)
      sm%rho_estimate = val
    case default
      write(0,*) 'Invalid choice for POLY_RHO_ESTIMATE, defaulting to amg_poly_rho_power'
      sm%variant = amg_poly_rho_est_power_
    end select      
  case('POLY_RHO_ESTIMATE_ITERATIONS')
    if (val>0) then 
      sm%rho_estimate_iterations = val
    else
      write(0,*) 'Invalid choice for POLY_RHO_ESTIMATE_ITERATIONS, defaulting to 20'
      sm%variant = 20
    end if
  case default
    call sm%amg_s_base_smoother_type%set(what,val,info,idx=idx)
  end select

  call psb_erractionrestore(err_act)
  return

9999 call psb_error_handler(err_act)

  return
end subroutine amg_s_poly_smoother_cseti
